#include <avr/io.h>
#include <avr/interrupt.h> /* for sei() */
#include <util/delay.h> /* for _delay_ms() */
#include <avr/pgmspace.h> /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_GET_TEMP 1
#define VENDOR_RQ_SET_FAN  0

uint8_t in_c(uint8_t pin)
{

DDRC &= ~(1<<pin);

PORTC |= (1<<pin);

return ((PINC & (1<<pin))>>pin); 
} 

void out_c(uint8_t pin, uint8_t val)
{

DDRC |= 1<<pin;

if (val)
PORTC |= 1<<pin;
else
PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
ADMUX = 0b01000000; 
ADMUX |= channel; 
ADCSRA = 0b11000110;


while ((ADCSRA & (1<<ADSC))) 
;
return ADCL + ADCH*256; 
}


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t *rq = (void *)data;
static uchar dataBuffer[2]; 


if (rq->bRequest == VENDOR_RQ_GET_TEMP)
{
uint16_t temp;
temp = read_adc(0);
dataBuffer[0] = temp/256;
dataBuffer[1] = temp%256;
usbMsgPtr = dataBuffer; 
return 2; 
}
if (rq->bRequest == VENDOR_RQ_SET_FAN)
    {
        uint8_t fan_no  = rq->wValue.bytes[0];
        uint8_t fan_val = rq->wValue.bytes[1];
        out_c(fan_no, fan_val);
        return 0;
    }

return 0; 
}



int main()
{
DDRC = 0b00010100; // PC0..PC2 = Output, PC3..PC5 = Input

PORTC = 0b00000000; // PC7..PC0 = 0

//PORTC |= (1 << 4); // Enable pull-up at PC3



usbInit();
usbDeviceDisconnect();
_delay_ms(300); 
usbDeviceConnect();
sei(); 
while (1) 
{ 
usbPoll(); 
}
return 0;
}
